{
 Designer: Craig Ward, 100554.2072@compuserve.com
 Date:     5/1/96
 Version:  3.4


 Function: Backup dialog DLL. User specificies the source and destination directories,
           then the dialog will copy all files.

 Update:   A couple of new calls have been added. For a full explanantion of each of the
           DLL's calls, please see below:

 Calls:
           procedure ShowBackupDlg(pSource,pDestination,pHelp,pCompat: pChar; iCompat: longint);
            - opens the dialog box. This proc copies the contents of a whole directory that
              conforms to the mask passed over.

           procedure HideBackupDlg(pSource,pDestination,pHelp,pCompat: pChar; iCompat: longint);
            - loads, but does not open dialog box, and executes the backup automatically, then closes.
              This proc copies the contents of a whole directory that conforms to the mask passed
              over.

           procedure CopyIndivFile(pSourceFile,pDestinationFile: pChar);
            - copies the source file to the target file.

           procedure ExpandIndivFile(pSourceFile,pDestinationFile: pChar);
            - copies the source file to the target file, and expands the file if it was
              compressed (using MS-COMPRESS.EXE).


 Extra:    Big thankyou to Dennis Passmore (71640.2464@compuserve.com), for his additions
           to the unit. These consisted of expanded error checking, creation of a custom type
           (the large buffer) to speed up the whole process and reduce stack usage, plus,
           the neat trick of changing the file-date of the backed-up files to reflect their
           backup date as opposed to their modified date.

           Also, a thankyou to Gregory Kraft (72114.3111@compuserve.com) who's enquiry
           precipitated the addition of the HideBackupDlg procedure.

           All criticisms, help and general advice is greatly welcomed.
*********************************************************************************}
unit Backup;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, StdCtrls, Buttons, FileCtrl, Gauges, ExtCtrls;

type
  TBackupDlg = class(TForm)
    DirList: TDirectoryListBox;
    FList: TFileListBox;
    Label1: TLabel;
    lblSource: TLabel;
    Label2: TLabel;
    lblDestination: TLabel;
    btnOK: TBitBtn;
    btnCancel: TBitBtn;
    Panel1: TPanel;
    Gauge1: TGauge;
    BitBtn1: TBitBtn;
    driveBox: TDriveComboBox;
    Bevel1: TBevel;
    Bevel2: TBevel;
    SpeedButton1: TSpeedButton;
    Bevel3: TBevel;
    Bevel4: TBevel;
    procedure btnCancelClick(Sender: TObject);
    procedure btnOKClick(Sender: TObject);
    procedure BitBtn1Click(Sender: TObject);
    procedure btnOKKeyPress(Sender: TObject; var Key: Char);
    procedure SpeedButton1Click(Sender: TObject);
  private
    { Private declarations }
    procedure CustInitialise(pSource,pDestination,pHelp,pCompat: pChar; iCompat: longint);
    procedure SetUpFiles;
    procedure CustCopyFiles(sSrce, sDest: string);
    procedure CustExpandFile(pSrce, pDest: pChar);
    function ChangeExt(sSrce: string): string;
  public
    { Public declarations }
  end;

var
  BackupDlg: TBackupDlg;
  iHlp: integer;

const
 iHelp: integer = 105; {help-context for SelectDirectory Dialog}


{exported procedures}
procedure ShowBackupDlg(pSource,pDestination,pHelp,pCompat: pChar; iCompat: longint); export;
procedure HideBackupDlg(pSource,pDestination,pHelp,pCompat: pChar; iCompat: longint); export;
procedure CopyIndivFile(pSourceFile,pDestinationFile: pChar); export;
procedure ExpandIndivFile(pSourceFile,pDestinationFile: pChar); export;

implementation

{$R *.DFM}

uses
 LZExpand;

{***custom routines*************************************************************}

{set environment}
procedure TBackupDlg.CustInitialise(pSource,pDestination,pHelp,pCompat: pChar; iCompat: longint);
begin

  {set environment}
  if Assigned(pSource) then
   lblSource.Caption := strPAS(pSource);
  if Assigned(pDestination) then
   lblDestination.Caption := strPAS(pDestination);
  if Assigned(pHelp) then
   Application.HelpFile := strPAS(pHelp);
  if Assigned(pCompat) then
   FList.Mask := StrPAS(pCompat);

  {check directories}
  if directoryExists(backupDlg.lblSource.caption) then
   begin
    FList.Directory := lblSource.caption;
    DirList.Directory := lblSource.Caption;
    DriveBox.Drive := DirList.Drive;
   end
  else
    messageDlg('Source Directory not found.',mtWarning,[mbOK],0);
  if not directoryExists(lblDestination.caption) then
   messageDlg('Destination Directory not found.',mtWarning,[mbOK],0);

end;

{***Exported Procedures*********************************************************}

{calls the dialog - invisible - and copies the file}
procedure CopyIndivFile(pSourceFile,pDestinationFile: pChar);
begin
 {create dialog}
 try
  BackupDlg := TBackupDlg.Create(application);

  if (StrPAS(pSourceFile) <> '') and (StrPAS(pDestinationFile) <> '') then
   BackupDlg.CustCopyFiles(StrPAS(pSourceFile),StrPAS(pDestinationFile));

 finally
  BackupDlg.Free;
 end;

end;

{calls the dialog - invisible - and copies the file, expanding it if compressed}
procedure ExpandIndivFile(pSourceFile,pDestinationFile: pChar);
begin
 {create dialog}
 try
  BackupDlg := TBackupDlg.Create(application);

  if (StrPAS(pSourceFile) <> '') and (StrPAS(pDestinationFile) <> '') then
   BackupDlg.CustExpandFile(pSourceFile,pDestinationFile);

 finally
  BackupDlg.Free;
 end;

end;


{calls the dialog - visible}
procedure ShowBackupDlg(pSource,pDestination,pHelp,pCompat: pChar; iCompat: longint);
begin
 {create dialog}
 try
  BackupDlg := TBackupDlg.Create(application);

  {initialise}
  BackupDlg.CustInitialise(pSource,pDestination,pHelp,pCompat,iCompat);

  {show}
  BackupDlg.ShowModal;

 finally
  BackupDlg.Free;
 end;

end;

{calls the dialog - invisible and runs backup automatically}
procedure HideBackupDlg(pSource,pDestination,pHelp,pCompat: pChar; iCompat: longint);
begin
 {create dialog}
 try
  BackupDlg := TBackupDlg.Create(application);

  {initialise}
  BackupDlg.CustInitialise(pSource,pDestination,pHelp,pCompat,iCompat);

  {execute copy proc}
  BackupDlg.SetupFiles;

 finally
  BackupDlg.Free;
 end;

end;


{***Buttons*********************************************************************}
{help}
procedure TBackupDlg.BitBtn1Click(Sender: TObject);
begin
 Application.HelpCommand(HELP_CONTEXT,BackupDlg.HelpContext);
end;


{close}
procedure TBackupDlg.btnCancelClick(Sender: TObject);
begin
 close;
end;

{copy}
procedure TBackupDlg.btnOKClick(Sender: TObject);
begin
 SetUpFiles;
end;

{Select Directory}
procedure TBackupDlg.SpeedButton1Click(Sender: TObject);
var
 sDir: ^string;
begin
 try
  New(sDir);

  sDir^ := lblDestination.Caption;
  if SelectDirectory(sDir^,[sdAllowCreate,sdPerformCreate,sdPrompt],iHelp) then
   lblDestination.caption := sDir^;

 finally
  Dispose(sDir)
 end;

end;


{***Copy procs******************************************************************}

{setup copying}
procedure TBackupDlg.SetUpFiles;
var
 OkToAll: boolean;
 iNum: integer;
 sSrce, sDest: ^string;
begin
 try
  New(sSrce);
  New(sDest);

  {initialise}
  OkToAll := false;
  iNum := 0;


 {ensure that source directory exists}
 if not directoryExists(lblSource.caption) then
  begin
   messageDlg('Source Directory not found.',mtWarning,[mbOK],0);
   exit;
  end;

 {ensure that destination directory exists}
 if not directoryExists(lblDestination.caption) then
  begin
   messageDlg('Destination Directory not found.',mtWarning,[mbOK],0);
   exit;
  end;

 {check that the user is not trying to copy over source files}
 if CompareStr(lblSource.Caption,lblDestination.Caption) = 0 then
  begin
   messageDlg('Can not overwrite source files.',mtWarning,[mbOK],0);
   exit;
  end;

 {ensure that there are items in the file-list box}
 if (FList.Items.Count) = 0 then
  begin
   messageDlg('No files to be copied.',mtWarning,[mbOK],0);
   exit;
  end;


 {now, safe to continue with copy...}

 Gauge1.MaxValue := FList.Items.Count;
 Panel1.Visible := True;

 {init for loop}
 for iNum := 0 to (FList.Items.Count -1) do
  begin
  {if either the source or destination had been specified as
   just a drive letter and a colan than a backslash would
   already exist so it must be removed. }
  sSrce^ := lblSource.caption;
  if sSrce^[length(sSrce^)]='\' then
    delete(sSrce^,length(sSrce^),1);
  sSrce^ := sSrce^ + '\' + (ExtractFileName(FList.Items.Strings[iNum]));

  sDest^ := lblDestination.caption;
  if sDest^[length(sDest^)]='\' then
    delete(sDest^,length(sDest^),1);
  sDest^ := sDest^ + '\' + (ExtractFileName(FList.Items.Strings[iNum]));

 {check to see if file exists}
 if not OkToAll then
  begin
  if FileExists(sDest^) then
   begin
    case messageDlg('Overwrite '+sDest^,mtConfirmation,[mbYes,mbAll,mbNo],0) of

      idYes:
       custCopyFiles(sSrce^,sDest^);

      (idNo+1): {mrAll}
       begin
        OkToAll := true;
        custCopyFiles(sSrce^,sDest^);
       end;

      idNo:
       {do nothing}

    end;
   end
  else
   {file doesn't already exist - so copy}
   custCopyFiles(sSrce^,sDest^);
  end
  else
   {file does already exist, but overwrite is true}
   custCopyFiles(sSrce^,sDest^);

  {update gauge}
   Gauge1.AddProgress(1);
   Application.ProcessMessages;

  end;

 {cleanup}
 Panel1.Visible := False;
 Gauge1.Progress := 0;
 OkToAll := false;

 finally
  Dispose(sSrce);
  Dispose(sDest);
 end;

end;


{copy routine}
procedure TBackupDlg.CustCopyFiles(sSrce,sDest: string);
type
  iobufptr = ^iobufr; {allowate a LARGE buffer to speed up copies}
  iobufr   = array[0..32767] of char; {MAX=65535}
var
 fSrce, fDest: file;
 wRead, wWritten: word;
 p: iobufptr;
 FDate: Longint;
begin

  {initialise}
  wRead := 0;
  wWritten := 0;

  {Set file access mode to readonly in case file is in use.}
  System.FileMode := fmOpenRead;
   {assign and open files}
   AssignFile(fSrce,sSrce);
   {$I-}
   Reset(fSrce,1);
   {$I+}
  {Set file access mode back to normal default for other processes}
  System.Filemode := fmOpenReadWrite;
  if IOResult <> 0  then
   begin
    messageDlg('Could not open: '+sSrce,mtWarning,[mbOK],0);
    exit;
   end;
  {Store file Date & Time for later use}
  FDate   := FileGetDate(TFileRec(fSrce).Handle);

  {Set file access mode to allow Exclusive Creation }
  System.Filemode := fmOpenWrite and fmShareExclusive;
   AssignFile(fDest,sDest);
   {$I-}
   Rewrite(fDest, 1);
   {$I+}
  {Set file access mode back to normal default for other processes}
  System.Filemode := fmOpenReadWrite;
  if IOResult <> 0  then
   begin
    {Close the Source file we already have open.}
    System.CloseFile(fSrce);
    messageDlg('Could not create: '+sDest,mtWarning,[mbOK],0);
    exit;
   end;

  {allocate a file iobuffer on Heap to avoid stack overflow error}
  new(p);

   {copy loop}
   repeat
    BlockRead(fSrce, p^, SizeOf(p^), wRead);
    BlockWrite(fDest, p^, wRead, wWritten);
   until (wRead = 0) or (wWritten <> wRead);

  { release heap space for iobuffer }
  dispose(p);

  {clean up}
  System.CloseFile(fSrce);
  System.CloseFile(fDest);

  {restore Source file date & time to Destination file }
  Reset(fDest,1);
  FileSetDate(TFileRec(fDest).Handle,FDate);
  System.CloseFile(fDest);

end;


{expansion routine}
procedure TbackupDlg.CustExpandFile(pSrce, pDest: pChar);
var
 iDest, iSrce: integer;
 tStruct: TOFStruct;
 p: pChar;
begin

 try
  p := StrAlloc(256);

  {change file extension}
  StrPCopy(p, ChangeExt( StrPAS(pDest) ) );

 {open source}
 iSrce := _lopen(pSrce,OF_SHARE_COMPAT);
 if iSrce = 0 then
  begin
   messageDlg('Could not create: '+StrPAS(pSrce),mtWarning,[mbOK],0);
   exit;
  end;

 {open target}
 iDest := LZOpenFile(p,tStruct,OF_CREATE);
 if iDest = 0 then
  begin
   messageDlg('Could not create: '+StrPAS(p),mtWarning,[mbOK],0);
   exit;
  end;

 {expand}
 if LZCopy(iSrce,iDest) < 0 then
  begin
   messageDlg('Could not expand: '+StrPAS(pSrce),mtWarning,[mbOK],0);
   exit;
  end;

 {close}
 if _lclose(iSrce) <> 0 then
  begin
   messageDlg('Could not close: '+StrPAS(pSrce),mtWarning,[mbOK],0);
   exit;
  end;

 {close}
 if _lclose(iDest) <> 0 then
  begin
   messageDlg('Could not close: '+StrPAS(p),mtWarning,[mbOK],0);
   exit;
  end;

 finally
  StrDispose(p);
 end;

end;

{change extension routine}
function TBackupDlg.ChangeExt(sSrce: string): string;
type
 {custom type - stores replacment extensions}
 TRepExtensions = record
  CurrExt: string;
  RepExt: string;
end;
var
 FExtensions: array[1..6] of TRepExtensions;
 iInc: integer;
 sExt: string[4];
begin

 {extensions - note: do not include ini files or databases since the user's version will always be newer}
 FExtensions[1].CurrExt := '.EX_';
 FExtensions[1].RepExt := '.EXE';

 FExtensions[2].CurrExt := '.DL_';
 FExtensions[2].RepExt := '.DLL';

 FExtensions[3].CurrExt := '.HL_';
 FExtensions[3].RepExt := '.HLP';

 FExtensions[4].CurrExt := '.BM_';
 FExtensions[4].RepExt := '.BMP';

 FExtensions[5].CurrExt := '.IC_';
 FExtensions[5].RepExt := '.ICO';

 FExtensions[6].CurrExt := '.RP_';
 FExtensions[6].RepExt := '.RPT';

 if sSrce[length(sSrce)] = '_' then
  begin
   sExt := ExtractFileExt(sSrce);
   {iterate through extensions}
   for iInc := 1 to 6 do
    begin
     if CompareText(FExtensions[iInc].CurrExt,sExt) = 0 then sExt := FExtensions[iInc].RepExt;
    end;
   {change extension}
   sSrce := ChangeFileExt(sSrce,sExt);
   result := sSrce;
  end
 else
  result := '';

end;


{***Designer's signature***********************************************}
procedure TBackupDlg.btnOKKeyPress(Sender: TObject; var Key: Char);
begin
 if Key = ^J then
  messageDlg('This was designed by Craig Ward. Craig Ward can be reached'+
             ' at 100554.2072@compuserve.com',mtInformation,[mbOK],0);
end;

{}
end.


